package com.acemondstudios.rootine.Database

import android.content.ContentValues
import android.content.Context
import android.database.Cursor
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.util.Log
import com.acemondstudios.rootine.Models.*
import com.acemondstudios.rootine.Utils.Utils
import java.text.SimpleDateFormat
import java.util.*


private var DATABASE_NAME = "rootine_db"
private var DATABASE_VERSION = 3

class DBEngine(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {


    override fun onCreate(db: SQLiteDatabase?) {
        // DATABASE TABLES CREATING

        Log.d("SQL", "Creating database...")

        if(db != null){
            // 1. USER
            var sql = "CREATE TABLE IF NOT EXISTS UserWeight (" +
                    "record_date DATE, " +
                    "weight NUMERIC, " +
                    "PRIMARY KEY(record_date) )"
            db.execSQL(sql)

            // 2. ROUTINE
            sql = "CREATE TABLE IF NOT EXISTS Routine (" +
                    "routine_id INTEGER, " +
                    "description TEXT, " +
                    "create_date DATE, " +
                    "PRIMARY KEY(routine_id) )"
            db.execSQL(sql)

            // 3. Day
            sql = "CREATE TABLE IF NOT EXISTS Day (" +
                    "day_id INTEGER, " +
                    "routine_id INTEGER, " +
                    "description TEXT, " +
                    "day_order INTEGER, " +
                    "done INTEGER, " +
                    "PRIMARY KEY(day_id) )"
            db.execSQL(sql)

            // 4. Category
            sql = "CREATE TABLE IF NOT EXISTS Category (" +
                    "category_id INTEGER, " +
                    "description TEXT," +
                    "PRIMARY KEY(category_id) )"
            db.execSQL(sql)

            // 5. Exercise
            sql = "CREATE TABLE IF NOT EXISTS Exercise (" +
                    "exercise_id INTEGER, " +
                    "day_id INTEGER, " +
                    "routine_id INTEGER, " +
                    "category_id INTEGER, " +
                    "exercise_order INTEGER, " +
                    "title TEXT, " +
                    "description TEXT, " +
                    "series INTEGER, " +
                    "mini INTEGER, " +
                    "maxi INTEGER, " +
                    "uid TEXT, " +
                    "PRIMARY KEY(exercise_id) )"
            db.execSQL(sql)

            // 6. Register
            sql = "CREATE TABLE IF NOT EXISTS Register (" +
                    "exercise_id INTEGER, " +
                    "date DATE, " +
                    "day_id INTEGER," +
                    "routine_id INTEGER," +
                    "series INTEGER," +
                    "mini INTEGER, " +
                    "maxi INTEGER, " +
                    "weight NUMERIC, " +
                    "transaction_id INTEGER, " +
                    "PRIMARY KEY(exercise_id, date) )"
            db.execSQL(sql)

            // 7. Images
            sql = "CREATE TABLE IF NOT EXISTS Images (" +
                    "uid TEXT, " +
                    "base64 BLOB, " +
                    "PRIMARY KEY(uid) )"
            db.execSQL(sql)

            Log.d("SQL", "Database has been created successfully")
        }
        else{
            return
        }
    }

    override fun onUpgrade(db: SQLiteDatabase?, oldVersion: Int, newVersion: Int) {
        if(oldVersion == 1 && newVersion == 2){
            // Format old routine dates to new format
            if(db != null) {
                val currDate = Utils.getCurrentStringDate()
                val sql = "UPDATE Routine set create_date = '$currDate'"
                val cursor: Cursor = db.rawQuery(sql, null)
                cursor.moveToFirst()
                cursor.close()
            }
        }
        else if(oldVersion == 2 && newVersion == 3){
            // Add new transaction_id column
            if(db != null) {
                val currDate = Utils.getCurrentStringDate()
                val sql = "ALTER TABLE Register ADD COLUMN transaction_id INTEGER;"
                val cursor: Cursor = db.rawQuery(sql, null)
                cursor.moveToFirst()
                cursor.close()

                db.delete("Register", null, null)
            }
        }
    }

    override fun onDowngrade(db: SQLiteDatabase?, oldVersion: Int, newVersion: Int) {

    }

    // ROUTINE METHODS

    fun addRoutine(routineName: String): Int{
        val mDb = writableDatabase
        var sql = "SELECT MAX(routine_id) FROM Routine"
        var currentIndex = 0
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentIndex = cursor.getInt(0)
        }
        cursor.close()
        currentIndex++

        val contentValues = ContentValues()
        contentValues.put("routine_id", currentIndex)
        contentValues.put("description", routineName)
        contentValues.put("create_date", Utils.getCurrentStringDate())
        mDb.insert("Routine", null, contentValues)
        return currentIndex
    }

    fun deleteRoutine(routineID: Int) {
        val mDb = writableDatabase
        mDb.delete("Routine", "routine_id=$routineID", null)

        // Delete all routine days
        val days = getAllDaysFromRoutine(routineID)
        for(day in days){
            deleteDay(day.day_id)
        }
    }

    fun getAllRoutines(db: SQLiteDatabase): ArrayList<Routine>{
        val list: ArrayList<Routine> = ArrayList()
        var sql = "SELECT routine_id, description, create_date FROM Routine;"
        val cursor: Cursor = db.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val calendar = Calendar.getInstance()
                val sdf = SimpleDateFormat("dd/MM/yyyy HH:mm:ss", Locale.getDefault())
                calendar.time = sdf.parse(cursor.getString(2))!!
                val routine = Routine(
                    cursor.getInt(0),
                    cursor.getString(1),
                    calendar
                )
                list.add(routine)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun getAllRoutines(): ArrayList<Routine>{
        val mDb = writableDatabase
        return getAllRoutines(mDb)
    }

    fun editRoutine(description: String, id: Number) {
        val mDb = writableDatabase

        val contentValues = ContentValues()
        contentValues.put("routine_id", id.toInt())
        contentValues.put("description", description)
        mDb.update("Routine", contentValues, "routine_id=$id", null)
    }

    fun routineHasRegisters(routine: Routine): Boolean{
        var hasExercises = false
        val days = getAllDaysFromRoutine(routine.routine_id)
        for(day in days){
            hasExercises = dayHasRegisters(day)
            if(hasExercises)
                break
        }
        return hasExercises
    }

    fun getRoutineHistory(): ArrayList<RoutineHistoryRegister>{
        val list: ArrayList<RoutineHistoryRegister> = ArrayList()
        val sql = "SELECT \n" +
                "\td.day_id, \n" +
                "\tr.routine_id, \n" +
                "\tr.description as routine_description, \n" +
                "\td.description as day_description, \n" +
                "\tMAX(reg.date) as date_done, \n" +
                "\treg.transaction_id\n" +
                "FROM Register reg\n" +
                "LEFT JOIN Day d ON d.day_id = reg.day_id\n" +
                "LEFT JOIN Routine r ON r.routine_id = reg.routine_id\n" +
                "GROUP BY transaction_id\n" +
                "ORDER BY MAX(reg.date) DESC"

        val db = writableDatabase
        val cursor: Cursor = db.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val calendar = Calendar.getInstance()
                val sdf = SimpleDateFormat("dd/MM/yyyy HH:mm:ss", Locale.getDefault())
                calendar.time = sdf.parse(cursor.getString(4))!!
                val routineHistoryRegister = RoutineHistoryRegister(
                    cursor.getInt(0),
                    cursor.getInt(1),
                    cursor.getString(2),
                    cursor.getString(3),
                    calendar,
                    cursor.getInt(5)
                )
                list.add(routineHistoryRegister)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun deleteRoutineHistoryRegister(routineHistoryRegister: RoutineHistoryRegister){
        val mDb = writableDatabase
        mDb.delete("Register", "transaction_id=${routineHistoryRegister.transaction_id}", null)
    }

    // END ROUTINE METHODS

    // DAY METHODS

    fun addDay(routineId: Number, dayName: String): Int {
        val mDb = writableDatabase

        var sql = "SELECT MAX(day_id) FROM Day"
        var currentIndex = 0
        var currentDayOrder = 0
        var cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentIndex = cursor.getInt(0)
        }
        cursor.close()
        currentIndex++


        sql = "SELECT MAX(day_order) FROM Day WHERE routine_id = $routineId"
        cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentDayOrder = cursor.getInt(0)
        }
        cursor.close()
        currentDayOrder++

        val contentValues = ContentValues()
        contentValues.put("day_id", currentIndex)
        contentValues.put("routine_id", routineId.toInt())
        contentValues.put("description", dayName)
        contentValues.put("day_order", currentDayOrder)
        contentValues.put("done", 0)
        mDb.insert("Day", null, contentValues)
        return currentIndex
    }

    fun getAllDaysFromRoutine(routineID: Int): ArrayList<Day>{
        val list: ArrayList<Day> = ArrayList()
        val mDb = writableDatabase
        var sql = "SELECT day_id, routine_id, description, day_order, done FROM Day WHERE routine_id = $routineID;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                var done = false
                if(cursor.getInt(4) == 1)
                    done = true
                val day = Day(
                    cursor.getInt(0),
                    cursor.getInt(1),
                    cursor.getString(2),
                    cursor.getInt(3),
                    done
                )
                list.add(day)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun editDay(description: String, day_id: Number) {
        val mDb = writableDatabase

        val contentValues = ContentValues()
        contentValues.put("day_id", day_id.toInt())
        contentValues.put("description", description)
        mDb.update("Day", contentValues, "day_id=$day_id", null)
    }

    fun deleteDay(dayID: Int) {
        val mDb = writableDatabase
        mDb.delete("Day", "day_id=$dayID", null)

        // Delete all day exercises
        val exercises = getExercises(dayID)
        for(exercise in exercises){
            deleteExercise(exercise)
        }
    }

    fun dayHasRegisters(day: Day): Boolean{
        var hasExercises = false
        val exercises = getExercises(day.day_id)
        for(exercise in exercises){
            hasExercises = exerciseHasRegisters(exercise)
            if(hasExercises)
                break
        }
        return hasExercises
    }

    fun markDayAsDone(dayID: Int) {
        val mDb = writableDatabase

        val contentValues = ContentValues()
        contentValues.put("done", 1)
        mDb.update("Day", contentValues, "day_id=$dayID", null)
    }

    fun resetRoutineDays(routineID: Int){
        val mDb = writableDatabase

        val contentValues = ContentValues()
        contentValues.put("done", 0)
        mDb.update("Day", contentValues, "routine_id=$routineID", null)
    }

    fun allDaysDone(routineID: Int): Boolean {
        val mDb = writableDatabase
        val sql = "SELECT COUNT(*) AS days,\n" +
                "(\n" +
                "    SELECT COUNT(*)\n" +
                "    FROM Day\n" +
                "    WHERE done = 1\n" +
                "    AND routine_id = $routineID\n" +
                ") AS done_days\n" +
                "FROM Day\n" +
                "WHERE routine_id = $routineID"
        var cursor: Cursor = mDb.rawQuery(sql, null)
        cursor = mDb.rawQuery(sql, null)
        var days = 0
        var doneDays = 0
        if(cursor.moveToFirst()){
            days = cursor.getInt(0)
            doneDays = cursor.getInt(1)
        }
        cursor.close()
        return doneDays == (days - 1)
    }

    // END DAY METHODS

    // EXERCISE METHODS

    fun addExercise(
        dayID: Int, routineID: Int, categoryID: Int, title: String,
        description: String, series: Int, min: Int, max: Int, base64: String
    ): Int {
        val mDb = writableDatabase

        var sql = "SELECT MAX(exercise_id) FROM Exercise"
        var currentIndex = 0
        var currentExerciseOrder = 0
        var cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentIndex = cursor.getInt(0)
        }
        cursor.close()
        currentIndex++

        // Add new base64 image
        var uid = ""
        if(base64.isNotEmpty()) {
           uid = this.createImage(base64)
        }

        sql = "SELECT MAX(exercise_order) FROM Exercise WHERE routine_id = $routineID AND day_id = $dayID"
        cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentExerciseOrder = cursor.getInt(0)
        }
        cursor.close()
        currentExerciseOrder++

        val contentValues = ContentValues()
        contentValues.put("exercise_id", currentIndex)
        contentValues.put("day_id", dayID)
        contentValues.put("routine_id", routineID)
        contentValues.put("category_id", categoryID)
        contentValues.put("exercise_order", currentExerciseOrder)
        contentValues.put("title", title)
        contentValues.put("description", description)
        contentValues.put("series", series)
        contentValues.put("mini", min)
        contentValues.put("maxi", max)
        contentValues.put("uid", uid)
        mDb.insert("Exercise", null, contentValues)
        return currentIndex
    }

    fun updateExercise(
        exerciseID: Int, dayID: Int, routineID: Int, categoryID: Int, title: String,
        description: String, series: Int, min: Int, max: Int, uid: String, base64: String
    ) {
        val mDb = writableDatabase

        var newUid = ""

        if(base64.isNotEmpty()) {
            if(uid.isNotEmpty()) {
                this.updateImage(uid, base64)
            }
            else {
                newUid = this.createImage(base64)
            }
        }

        val contentValues = ContentValues()
        contentValues.put("category_id", categoryID)
        contentValues.put("title", title)
        contentValues.put("description", description)
        contentValues.put("series", series)
        contentValues.put("mini", min)
        contentValues.put("maxi", max)
        if(newUid.isNotEmpty()){
            contentValues.put("uid", newUid)
        }
        mDb.update(
            "Exercise",
            contentValues,
            "exercise_id=$exerciseID AND day_id=$dayID AND routine_id=$routineID",
            null
        )
        deleteCategoriesWithoutExercises()
    }

    fun getExercises(dayID: Int): ArrayList<Exercise> {
        val categoriesMap = getAllCategoriesDict()

        val list: ArrayList<Exercise> = ArrayList()
        val mDb = writableDatabase
        var sql = "SELECT exercise_id, day_id, routine_id, category_id, exercise_order, title, description, series, mini, maxi, uid FROM Exercise WHERE day_id = $dayID "
        sql += "ORDER BY category_id, exercise_order;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val categoryID = cursor.getInt(3)
                val exercise = Exercise(
                    cursor.getInt(0),
                    cursor.getInt(1),
                    cursor.getInt(2),
                    categoriesMap[categoryID] as Category,
                    cursor.getInt(4),
                    cursor.getString(5),
                    cursor.getString(6),
                    cursor.getInt(7),
                    cursor.getInt(8),
                    cursor.getInt(9),
                    cursor.getString(10)
                )
                list.add(exercise)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun deleteExercise(exercise: Exercise) {
        val mDb = writableDatabase
        var contentValues = ContentValues()
        val exerciseID = exercise.exercise_id
        mDb.delete("Exercise", "exercise_id=$exerciseID", null)

        deleteCategoryIfDoesNotHaveExercises(exercise.category.category_id)

        // Delete all registers of this exercise

        contentValues = ContentValues()
        contentValues.put("exercise_id", exerciseID)
        mDb.delete("Register", "exercise_id=$exerciseID", null)

        // Delete exercise image

        this.deleteImage(exercise.uid)
    }

    fun exerciseHasRegisters(exercise: Exercise): Boolean{
        val mDb = writableDatabase
        val exerciseID = exercise.exercise_id
        val sql = "SELECT COUNT(*) FROM Register WHERE exercise_id = $exerciseID "

        val cursor: Cursor = mDb.rawQuery(sql, null)
        var registers = 0
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                registers = cursor.getInt(0)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return registers != 0
    }

    // END EXERCISE METHODS

    // CATEGORY METHODS

    fun getAllCategories(): ArrayList<Category> {
        val list: ArrayList<Category> = ArrayList()
        val mDb = writableDatabase
        var sql = "SELECT category_id, description FROM Category ORDER BY description;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val category = Category(
                    cursor.getInt(0),
                    cursor.getString(1)
                )
                list.add(category)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun getAllCategoriesDict(): HashMap<Int, Category> {
        val categories: ArrayList<Category> = getAllCategories()
        val map = hashMapOf<Int, Category>()

        for(category in categories){
            map[category.category_id] = category
        }

        return map
    }

    fun getExerciseSpinnerModels(): ArrayList<ExerciseSpinnerModel> {
        val sql = "SELECT ex.exercise_id, c.description, ex.title\n" +
                "FROM Exercise ex\n" +
                "LEFT JOIN Category c ON c.category_id = ex.category_id\n" +
                "ORDER BY c.category_id"
        val list: ArrayList<ExerciseSpinnerModel> = ArrayList()
        val mDb = writableDatabase
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val exerciseSpinnerModel = ExerciseSpinnerModel(
                    cursor.getInt(0),
                    cursor.getString(1) + " - " + cursor.getString(2)
                )
                list.add(exerciseSpinnerModel)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun addCategory(description: String): Int {
        val mDb = writableDatabase

        var sql = "SELECT MAX(category_id) FROM Category"
        var currentIndex = 0
        var cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            currentIndex = cursor.getInt(0)
        }
        cursor.close()
        currentIndex++

        val contentValues = ContentValues()
        contentValues.put("category_id", currentIndex)
        contentValues.put("description", description)
        mDb.insert("Category", null, contentValues)
        return currentIndex
    }

    fun deleteCategory(categoryID: Int) {
        val mDb = writableDatabase
        mDb.delete("Category", "category_id=$categoryID", null)
    }

    fun deleteCategoryIfDoesNotHaveExercises(categoryID: Int){
        // Delete categories that doesn't have any exercise
        val mDb = writableDatabase
        val sql = "SELECT COUNT(*) FROM Exercise WHERE category_id = $categoryID "
        val cursor: Cursor = mDb.rawQuery(sql, null)
        var exercices = 0
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                exercices = cursor.getInt(0)
                cursor.moveToNext()
            }
        }
        if(exercices == 0){
            deleteCategory(categoryID)
        }
        cursor.close()
    }

    fun deleteCategoriesWithoutExercises(){
        val categories = getAllCategories()
        for(category in categories){
            deleteCategoryIfDoesNotHaveExercises(category.category_id)
        }
    }

    // END CATEGORY METHODS

    // REGISTER METHODS

    fun saveRegisters(registers: ArrayList<Register>){
        val mDb = writableDatabase
        val transactionID = getTransactionID()
        for(register in registers){
            val contentValues = ContentValues()
            contentValues.put("exercise_id", register.exercise_id)
            contentValues.put("date", Utils.getStringDate(register.date))
            contentValues.put("day_id", register.day_id)
            contentValues.put("routine_id", register.routine_id)
            contentValues.put("series", register.series)
            contentValues.put("mini", register.min)
            contentValues.put("maxi", register.max)
            contentValues.put("weight", register.weight)
            contentValues.put("transaction_id", transactionID)
            mDb.insert("Register", null, contentValues)
        }
    }

    fun getLastWeight(exerciseID: Int, dayID: Int, routineID: Int): Double{
        val mDb = writableDatabase
        var weight = 0.0
        var sql = "SELECT weight FROM Register WHERE exercise_id = $exerciseID AND day_id = $dayID and routine_id = $routineID ORDER BY date DESC LIMIT 1;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                weight = cursor.getDouble(0)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return weight
    }

    fun getLastSeries(exerciseID: Int, dayID: Int, routineID: Int): String{
        val mDb = writableDatabase
        var series = 0
        var min = 0
        var max = 0
        var sql = "SELECT series, mini, maxi FROM Register WHERE exercise_id = $exerciseID AND day_id = $dayID and routine_id = $routineID ORDER BY date DESC LIMIT 1;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                series = cursor.getInt(0)
                min = cursor.getInt(1)
                max = cursor.getInt(2)
                cursor.moveToNext()
            }
        }
        cursor.close()
        var res = ""
        if(series != 0 && min != 0 && max != 0) {
            res = "$series x $min"
            if (min != max) {
                res += "-$max"
            }
        }
        return res
    }

    private fun getTransactionID(): Int {
        val mDb = writableDatabase
        var transactionID = 0
        var sql = "SELECT IFNULL(MAX(transaction_id) + 1, 0) as next_id from Register;"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if (cursor.moveToFirst()) {
            while (!cursor.isAfterLast) {
                transactionID = cursor.getInt(0)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return transactionID
    }

    // END REGISTER METHODS

    // IMAGES METHODS

    private fun generateUUID(): String {
        return UUID.randomUUID().toString().replace("-", "", false).toUpperCase(Locale.ROOT)
    }

    private fun createImage(base64: String): String{
        val mDb = writableDatabase
        var uid = generateUUID()
        val contentValues = ContentValues()
        contentValues.put("uid", uid)
        contentValues.put("base64", base64)
        mDb.insert("Images", null, contentValues)
        return uid
    }

    fun readImage(uid: String): String {
        val mDb = writableDatabase
        var base64 = ""
        var sql = "SELECT base64 FROM Images WHERE uid = '$uid'"
        val cursor: Cursor = mDb.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                base64 = cursor.getString(0)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return base64
    }

    private fun updateImage(uid: String, base64: String) {
        val mDb = writableDatabase
        if(uid.isNotEmpty()) {
            val contentValues = ContentValues()
            contentValues.put("base64", base64)
            mDb.update(
                "Images",
                contentValues,
                "uid='$uid'",
                null
            )
        }
    }

    fun deleteImage(uid: String) {
        val mDb = writableDatabase
        var contentValues = ContentValues()
        if(uid.isNotEmpty()) {
            mDb.delete("Images", "uid='$uid'", null)

            contentValues.put("uid", "")
            mDb.update(
                "Exercise",
                contentValues,
                "uid='$uid'",
                null
            )
        }
    }

    fun deleteAllImages(){
        // For firefighting
        val mDb = writableDatabase
        mDb.delete("Images", null, null)
        var contentValues = ContentValues()
        contentValues.put("uid", "")
        mDb.update(
            "Exercise",
            contentValues,
            null,
            null
        )
    }

    // END IMAGES METHODS

    // USER WEIGHT METHODS

    fun createUserWeight(weight: Double){
        val mDb = writableDatabase
        val contentValues = ContentValues()
        contentValues.put("record_date", Utils.getCurrentStringDate())
        contentValues.put("weight", weight)
        mDb.insert("UserWeight", null, contentValues)
    }

    fun getAllUserWeight(): ArrayList<UserWeight>{
        val db = writableDatabase
        val list: ArrayList<UserWeight> = ArrayList()
        var sql = "SELECT * FROM (SELECT record_date, weight FROM UserWeight ORDER BY record_date DESC LIMIT 20) order by record_date ASC;"
        val cursor: Cursor = db.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val calendar = Calendar.getInstance()
                val sdf = SimpleDateFormat("dd/MM/yyyy HH:mm:ss", Locale.getDefault())
                calendar.time = sdf.parse(cursor.getString(0))!!
                val userWeight = UserWeight(
                    calendar,
                    cursor.getDouble(1)
                )
                list.add(userWeight)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun getAllExerciseWeightRecords(exerciseID: Int): ArrayList<UserWeight>{
        val sql = "SELECT *\n" +
                "FROM\n" +
                "(SELECT date, weight\n" +
                "FROM Register\n" +
                "WHERE exercise_id = $exerciseID\n" +
                "ORDER BY date DESC LIMIT 20)\n" +
                "ORDER BY date ASC"
        val db = writableDatabase
        val list: ArrayList<UserWeight> = ArrayList()
        val cursor: Cursor = db.rawQuery(sql, null)
        if(cursor.moveToFirst()){
            while(!cursor.isAfterLast){
                val calendar = Calendar.getInstance()
                val sdf = SimpleDateFormat("dd/MM/yyyy HH:mm:ss", Locale.getDefault())
                calendar.time = sdf.parse(cursor.getString(0))!!
                val userWeight = UserWeight(
                    calendar,
                    cursor.getDouble(1)
                )
                list.add(userWeight)
                cursor.moveToNext()
            }
        }
        cursor.close()
        return list
    }

    fun updateUserWeight(record_date: String, oldWeight: Double, newWeight: Double) {
        val mDb = writableDatabase
        val contentValues = ContentValues()
        contentValues.put("weight", newWeight)
        mDb.update(
            "UserWeight",
            contentValues,
            "record_date='$record_date' AND weight = '$oldWeight'",
            null
        )
    }

    fun deleteUserWeight(record_date: String, weight: Double) {
        val mDb = writableDatabase
        mDb.delete("UserWeight", "record_date='$record_date' AND weight='$weight'", null)
    }

    // END USER WEIGHT METHODS
}